@@ -15,11 +15,19 @@
 # limitations under the License.
 
 import abc
+import base64
 import functools
+import subprocess
 
 import urllib.parse as urlparse
 
+from oslo_log import log
+from tempest import config
 from tempest.lib.common import rest_client
+from tempest.lib.common import ssh
+
+CONF = config.CONF
+LOG = log.getLogger(__name__)
 
 
 def handle_errors(f):
@@ -157,7 +165,7 @@ class BaseClient(rest_client.RestClient, metaclass=abc.ABCMeta):
 
         :param resource: The name of the REST resource, e.g., 'audits'.
         :param uuid: The unique identifier of an object in UUID format.
-        :return: A tuple with the server response and the response body.
+        :returns: A tuple with the server response and the response body.
         """
 
         uri = self._get_uri(resource, uuid)
@@ -171,7 +179,7 @@ class BaseClient(rest_client.RestClient, metaclass=abc.ABCMeta):
 
         :param resource: The name of the REST resource, e.g., 'audits'.
         :param uuid: The unique identifier of an object in UUID format.
-        :return: A tuple with the server response and the serialized patched
+        :returns: A tuple with the server response and the serialized patched
                  object.
         """
 
@@ -207,3 +215,103 @@ class BaseClient(rest_client.RestClient, metaclass=abc.ABCMeta):
         resp, body = self.put(uri, body=put_body)
         self.expected_success(202, int(resp['status']))
         return resp, body
+
+
+class BaseCmdClient(metaclass=abc.ABCMeta):
+
+    @abc.abstractmethod
+    def exec_cmd(self, cmd, input_data=None, timeout=None):
+        """Execute a command with an optional input data.
+
+        :param cmd: command to be execute, which can be a string
+            or a sequence of arguments.
+        :param input_data: data to be sent to process stdin
+        :param timeout: communication timeout in seconds
+        :return: output written to stdout.
+        :raises: Exception when command fails.
+        """
+        pass
+
+
+class SubProcessCmdClient(BaseCmdClient):
+    """Command execution client based on subprocess"""
+
+    def exec_cmd(self, cmd, input_data=None, timeout=None):
+        """Execute a command with an optional input data.
+
+        :param cmd: command to be execute, which can be a string
+          or a sequence of arguments.
+        :param input_data: data to be sent to process stdin
+        :param timeout: communication timeout in seconds
+        :return: output written to stdout.
+        :raises: Exception when command fails.
+        """
+        LOG.debug(f"Executing command '{cmd}' with input data '{input_data}'")
+        sp = subprocess.Popen(cmd,
+                              stdout=subprocess.PIPE,
+                              stdin=subprocess.PIPE,
+                              stderr=subprocess.PIPE, text=True)
+        out, err = sp.communicate(input=input_data, timeout=timeout)
+
+        if len(err) > 1:
+            raise Exception(f"Subprocess command failed with: {err}")
+
+        return out
+
+
+class SshCmdClient(BaseCmdClient, ssh.Client):
+
+    def __init__(self, host, username, password=None, timeout=300, pkey=None,
+                 channel_timeout=10, look_for_keys=True, key_filename=None,
+                 port=22, pkey_type='rsa'):
+
+        # Prefix to be always include in all commands
+        self._cmd_prefix = ""
+
+        super(SshCmdClient, self).__init__(
+            host=host, username=username, password=password, timeout=timeout,
+            pkey=pkey, channel_timeout=channel_timeout,
+            look_for_keys=look_for_keys, key_filename=key_filename, port=port,
+            ssh_key_type=pkey_type)
+
+    @property
+    def cmd_prefix(self):
+        return self._cmd_prefix
+
+    @cmd_prefix.setter
+    def cmd_prefix(self, value):
+        self._cmd_prefix = value
+
+    def exec_cmd(self, cmd, input_data=None, timeout=None):
+        """Execute a command with an optional input data.
+
+        :param cmd: command to be execute, which can be a string
+            or a sequence of arguments.
+        :param input_data: data to be sent to process stdin
+        :param timeout: communication timeout in seconds
+
+        :returns: output written to stdout.
+        :raises: Exception when command fails.
+        """
+        cmd_str = " ".join(cmd) if isinstance(cmd, list) else cmd
+
+        if input_data:
+            # if stdin data, encode to base64 and decode in the remote host
+            b64_input_str = base64.b64encode(
+                input_data.encode("ascii")).decode("ascii")
+            cmd_str = (
+                f"sh -c \"echo \"{b64_input_str}\" | base64 -d | {cmd_str}\"")
+
+        if self.cmd_prefix:
+            cmd_str = f"{self.cmd_prefix} {cmd_str}"
+
+        LOG.debug(f"Executing command '{cmd_str}' on host '{self.host}'")
+
+        if timeout:
+            original_timeout = self.timeout
+            self.timeout = timeout
+        try:
+            return super(SshCmdClient, self).exec_command(cmd=cmd_str)
+        finally:
+            if timeout:
+                self.timeout = original_timeout
