@@ -16,33 +16,51 @@
 # limitations under the License.
 #
 
+import base64
 import functools
 import json
+import os_traits
 import random
+import textwrap
 import time
 
 from datetime import datetime
 from datetime import timedelta
-import os_traits
+from datetime import timezone
+
 from oslo_log import log
 from tempest.common import waiters
 from tempest import config
 from tempest.lib.common import api_microversion_fixture
+from tempest.lib.common import api_version_utils
 from tempest.lib.common.utils import data_utils
 from tempest.lib.common.utils import test_utils
 from tempest.lib import exceptions
 from tempest.scenario import manager
 
 from watcher_tempest_plugin import infra_optim_clients as clients
+from watcher_tempest_plugin.services.infra_optim.v1.json import (
+    api_microversion_fixture as watcher_microversion_fixture
+)
 
 
 LOG = log.getLogger(__name__)
 CONF = config.CONF
 
+# Minimal Nova API required to specify the hosts used when creating
+# instances is 2.74.This constant will be used to set the minimal
+# version of nova to be used in tests which use create_one_instance_per_host.
+# Note that it will not affect the version of the Nova API used by
+# Watcher, only the version used by tempest when used from those tests.
+NOVA_API_VERSION_CREATE_WITH_HOST = '2.74'
+
 
 class BaseInfraOptimScenarioTest(manager.ScenarioTest):
     """Base class for Infrastructure Optimization API tests."""
 
+    min_microversion = None
+    max_microversion = manager.LATEST_MICROVERSION
+
     # States where the object is waiting for some event to perform a transition
     IDLE_STATES = ('RECOMMENDED', 'FAILED', 'SUCCEEDED', 'CANCELLED')
     # States where the object can only be DELETED (end of its life-cycle)
@@ -56,6 +74,7 @@ class BaseInfraOptimScenarioTest(manager.ScenarioTest):
     PROMETHEUS_METRIC_MAP = dict(
         host_cpu_usage='node_cpu_seconds_total',
         host_ram_usage='node_memory_MemAvailable_bytes',
+        host_ram_total='node_memory_MemTotal_bytes',
         instance_cpu_usage='ceilometer_cpu',
         instance_ram_usage='ceilometer_memory_usage')
 
@@ -75,10 +94,16 @@ class BaseInfraOptimScenarioTest(manager.ScenarioTest):
 
     @classmethod
     def skip_checks(cls):
-        super(manager.ScenarioTest, cls).skip_checks()
+        super(BaseInfraOptimScenarioTest, cls).skip_checks()
         if not CONF.service_available.watcher:
             raise cls.skipException('Watcher support is required')
 
+        api_version_utils.check_skip_with_microversion(
+            cls.min_microversion,
+            cls.max_microversion,
+            CONF.optimize.min_microversion,
+            CONF.optimize.max_microversion)
+
     @classmethod
     def setup_credentials(cls):
         cls._check_network_config()
@@ -92,18 +117,26 @@ class BaseInfraOptimScenarioTest(manager.ScenarioTest):
         cls.gnocchi = cls.mgr.gn_client
         cls.placement_client = cls.mgr.placement_client
         cls.prometheus_client = cls.mgr.prometheus_client
+        cls.flavors_client = cls.mgr.flavors_client
 
     def setUp(self):
         super(BaseInfraOptimScenarioTest, self).setUp()
         self.useFixture(api_microversion_fixture.APIMicroversionFixture(
-            compute_microversion=CONF.compute.min_microversion))
+            compute_microversion=self.compute_request_microversion))
         self.useFixture(api_microversion_fixture.APIMicroversionFixture(
             placement_microversion=CONF.placement.min_microversion))
+        self.useFixture(watcher_microversion_fixture.APIMicroversionFixture(
+            optimize_microversion=self.request_microversion))
 
     @classmethod
     def resource_setup(cls):
         super(BaseInfraOptimScenarioTest, cls).resource_setup()
 
+        cls.request_microversion = (
+            api_version_utils.select_request_microversion(
+                cls.min_microversion,
+                CONF.optimize.min_microversion))
+
     @classmethod
     def resource_cleanup(cls):
         """Ensure that all created objects get destroyed."""
@@ -116,6 +149,17 @@ class BaseInfraOptimScenarioTest(manager.ScenarioTest):
             detail=True)['hypervisors']
         return hypervisors
 
+    @classmethod
+    def get_hypervisor_details(cls, node_name):
+        """Get hypervisor details by node name."""
+        hypervisors = cls.get_hypervisors_setup()
+        for hyp in hypervisors:
+            if hyp['hypervisor_hostname'] == node_name:
+                return hyp
+        raise exceptions.InvalidConfiguration(
+            "Hypervisor %s not found in the list of hypervisors." %
+            node_name)
+
     @classmethod
     def get_compute_nodes_setup(cls):
         services_client = cls.mgr.services_client
@@ -124,6 +168,33 @@ class BaseInfraOptimScenarioTest(manager.ScenarioTest):
         return [srv for srv in available_services
                 if srv.get('binary') == 'nova-compute']
 
+    @classmethod
+    def get_enabled_compute_nodes(cls):
+        cls.initial_compute_nodes_setup = cls.get_compute_nodes_setup()
+        return [cn for cn in cls.initial_compute_nodes_setup
+                if cn.get('status') == 'enabled']
+
+    @classmethod
+    def get_host_other_than(cls, server_id):
+        source_host = cls.get_host_for_server(server_id)
+
+        svcs = cls.os_admin.services_client.list_services(
+            binary='nova-compute')['services']
+        hosts = []
+        for svc in svcs:
+            if CONF.compute.target_hosts_to_avoid in svc['host']:
+                continue
+            if svc['state'] == 'up' and svc['status'] == 'enabled':
+                if CONF.compute.compute_volume_common_az:
+                    if svc['zone'] == CONF.compute.compute_volume_common_az:
+                        hosts.append(svc['host'])
+                else:
+                    hosts.append(svc['host'])
+
+        for target_host in hosts:
+            if source_host != target_host:
+                return target_host
+
     @classmethod
     def wait_for_compute_node_setup(cls):
 
@@ -207,7 +278,8 @@ class BaseInfraOptimScenarioTest(manager.ScenarioTest):
 
     def _live_migrate(self, server_id, target_host, state):
         self._migrate_server_to(server_id, target_host)
-        waiters.wait_for_server_status(self.servers_client, server_id, state)
+        waiters.wait_for_server_status(self.os_admin.servers_client,
+                                       server_id, state)
         migration_list = (self.mgr.migrations_client.list_migrations()
                           ['migrations'])
 
@@ -222,14 +294,37 @@ class BaseInfraOptimScenarioTest(manager.ScenarioTest):
 
         self.assertEqual(target_host, server_host, msg)
 
-    def _create_one_instance_per_host_with_statistic(self, metrics=dict()):
-        """Create 1 instance per compute node and make instance statistic
+    def _create_custom_flavor(self, ram=512, vcpus=1):
+        """Create a flavor with custom RAM size
+
+        :param ram: RAM in MB to be set for the flavor.
+        :returns: A flavor id
+        """
+
+        flavor_id = self.flavors_client.create_flavor(
+            name=data_utils.rand_name('watcher_flavor'),
+            ram=ram,
+            vcpus=vcpus,
+            disk=1,
+            ephemeral=0,
+            swap=0,
+            rxtx_factor=1.0,
+            is_public=True)['flavor']['id']
+        self.addCleanup(test_utils.call_and_ignore_notfound_exc,
+                        self.flavors_client.delete_flavor, flavor_id)
+        return flavor_id
+
+    def _create_one_instance_per_host(self, flavor=None, run_command=None):
+        """Create one instance per compute node
 
         This goes up to the min_compute_nodes threshold so that things don't
         get crazy if you have 1000 compute nodes but set min to 3.
 
-        :param metrics: The metrics add to resource when using Gnocchi
+        :param flavor: Flavor Name or ID
+        :param run_command: the command you want to run in the new instances
+        :returns: A list of instance UUIDs.
         """
+
         compute_nodes = self.get_compute_nodes_setup()
         instances = self.mgr.servers_client.list_servers(
             detail=True)['servers']
@@ -238,6 +333,7 @@ class BaseInfraOptimScenarioTest(manager.ScenarioTest):
 
         hypervisors = self.get_hypervisors_setup()
         created_instances = []
+
         for node in compute_nodes[:CONF.compute.min_compute_nodes]:
             hyp_id = [
                 hyp['id'] for hyp in hypervisors
@@ -260,25 +356,51 @@ class BaseInfraOptimScenarioTest(manager.ScenarioTest):
             self.assertNotEqual(0, retry)
             # by getting to active state here, this means this has
             # landed on the host in question.
-            instance = self.create_server(image_id=CONF.compute.image_ref,
-                                          wait_until='ACTIVE',
-                                          clients=self.os_primary)
-            # get instance object again as admin
-            instance = self.mgr.servers_client.show_server(
-                instance['id'])['server']
+            instance = self._create_instance(node['host'], flavor, run_command)
             created_instances.append(instance)
-            self.make_instance_statistic(instance, metrics=metrics)
-
         return created_instances
 
+    def _create_instance(self, host=None, flavor=None, run_command=None):
+        # We enforce the compute node where we create the instance to
+        # make sure we have one node on each compute.
+        # This requires Nova API version 2.74 or higher.
+        kwargs_server = {'host': host} if host else {}
+        validatable = False
+        validation_resources = None
+        if run_command:
+            # In case we want to run commands we will be injecting it via
+            # user_data which requires to setup the instance as validatable
+            # in tempest.common.compute.create_test_server
+            validation_resources = self.get_test_validation_resources(
+                self.os_admin)
+            validatable = True
+            script = '''
+                    #!/bin/sh
+                    {run_command}
+                    '''.format(run_command=run_command)
+            script_clean = textwrap.dedent(script).lstrip().encode('utf8')
+            script_b64 = base64.b64encode(script_clean)
+            kwargs_server['user_data'] = script_b64
+        flavor = flavor if flavor else CONF.compute.flavor_ref
+        instance = self.create_server(
+            image_id=CONF.compute.image_ref, wait_until='ACTIVE',
+            flavor=flavor, clients=self.os_admin, validatable=validatable,
+            validation_resources=validation_resources,
+            **kwargs_server)
+        # get instance object again as admin
+        instance = self.mgr.servers_client.show_server(
+            instance['id'])['server']
+        return instance
+
     def _pack_all_created_instances_on_one_host(self, instances):
         hypervisors = [
             hyp['hypervisor_hostname'] for hyp in self.get_hypervisors_setup()
             if hyp['state'] == 'up']
         node = hypervisors[0]
         for instance in instances:
-            if instance.get('OS-EXT-SRV-ATTR:hypervisor_hostname') != node:
+            if self.get_host_for_server(instance['id']) != node:
                 self._live_migrate(instance['id'], node, 'ACTIVE')
+        return node
 
     # ### GNOCCHI ### #
 
@@ -295,11 +417,14 @@ class BaseInfraOptimScenarioTest(manager.ScenarioTest):
             resp, body = self.gnocchi.search_resource(**search_body)
             body = body[0]
             for metric_name in kwargs['metrics'].keys():
-                metric_body = {
+                default_metric = {
                     "archive_policy_name": "low",
                     "resource_id": body['id'],
-                    "name": metric_name
+                    "name": metric_name,
+                    "unit": "ns"
                 }
+                metric_body = {**default_metric,
+                               **kwargs['metrics'][metric_name]}
                 if body['metrics'].get(metric_name, None):
                     self.gnocchi.delete_metric(body['metrics'][metric_name])
                 self.gnocchi.create_metric(**metric_body)
@@ -307,33 +432,81 @@ class BaseInfraOptimScenarioTest(manager.ScenarioTest):
             body = body[0]
         return resp, body
 
-    def _make_measures(self, measures_count, time_step):
+    def _make_measures_host(self, measures_count, time_step, min=10, max=20):
         measures_body = []
-        for i in range(1, measures_count + 1):
-            dt = datetime.utcnow() - timedelta(minutes=i * time_step)
+        now = datetime.now(timezone.utc)
+        for i in range(0, measures_count):
+            dt = now - timedelta(minutes=i * time_step)
             measures_body.append(
-                dict(value=random.randint(10, 20),
+                dict(value=random.randint(int(min), int(max)),
                      timestamp=dt.replace(microsecond=0).isoformat())
             )
         return measures_body
 
-    def make_host_statistic(self, metrics=dict()):
+    def _make_measures_instance(self, measures_count, time_step,
+                                min=80, max=90, metric_type='cpu'):
+        measures_body = []
+        now = datetime.now(timezone.utc)
+
+        if metric_type == "cpu":
+            final_cpu = (measures_count + 1) * 60 * time_step * 1e9
+            for i in range(0, measures_count):
+                dt = now - timedelta(minutes=i * time_step)
+                cpu = final_cpu - ((i - 1) * 60 * time_step * 1e9
+                                   * random.randint(int(min), int(max)) / 100)
+                measures_body.append(
+                    dict(value=cpu,
+                         timestamp=dt.replace(microsecond=0).isoformat())
+                )
+        elif metric_type == "ram":
+            for i in range(0, measures_count):
+                dt = now - timedelta(minutes=i * time_step)
+                ram = random.randint(int(min), int(max))
+                measures_body.append(
+                    dict(value=ram,
+                         timestamp=dt.replace(microsecond=0).isoformat())
+                )
+        else:
+            raise ValueError(f"Unsupported metric_type: {metric_type}")
+
+        return measures_body
+
+    def clean_injected_metrics(self):
+        """Delete all injected metrics from datastore.
+
+        This is useful to ensure that the tests are not affected by
+        previously injected metrics.
+        """
+        LOG.debug("Deleting injected metrics from Datastore")
+        if CONF.optimize.datasource == "gnocchi":
+            # TODO(morenod): Add function for deleting injected metrics
+            # from Gnocchi.
+            pass
+        elif CONF.optimize.datasource == "prometheus":
+            self.prometheus_client.delete_series()
+
+    def make_host_statistic(self, metrics=dict(), loaded_hosts=[]):
         """Add host metrics to the datasource
 
         :param metrics: Metrics that should be created
           in the configured datasource.
+        :param loaded_hosts: list of hosts that we want to inject data
+          representing high usage of resource.
         """
         # data sources that support fake metrics
         if CONF.optimize.datasource == "gnocchi":
-            self.make_host_statistic_gnocchi(metrics)
+            self.make_host_statistic_gnocchi(metrics,
+                                             loaded_hosts=loaded_hosts)
         elif CONF.optimize.datasource == "prometheus":
-            self.make_host_statistic_prometheus()
+            self.make_host_statistic_prometheus(loaded_hosts=loaded_hosts)
 
-    def make_host_statistic_gnocchi(self, metrics=dict()):
+    def make_host_statistic_gnocchi(self, metrics=dict(), loaded_hosts=[]):
         """Create host resource and its measures
 
         :param metrics: Metrics that should be created
           in the Gnocchi datasource.
+        :param loaded_hosts: list of hosts that we want to inject data
+          representing high usage of resource.
         """
         hypervisors_client = self.mgr.hypervisor_client
         hypervisors = hypervisors_client.list_hypervisors(
@@ -342,6 +515,9 @@ class BaseInfraOptimScenarioTest(manager.ScenarioTest):
             metrics = {
                 self.GNOCCHI_METRIC_MAP['host_cpu_usage']: {
                     'archive_policy_name': 'low'
+                },
+                self.GNOCCHI_METRIC_MAP['host_ram_usage']: {
+                    'archive_policy_name': 'low'
                 }
             }
         else:
@@ -360,11 +536,33 @@ class BaseInfraOptimScenarioTest(manager.ScenarioTest):
                 'id': host_name
             }
             _, res = self.create_resource(**resource_params)
-            metric_uuid = res['metrics'][
+
+            # Generate host_cpu_usage fake metrics
+            cpu_metric_uuid = res['metrics'][
                 self.GNOCCHI_METRIC_MAP['host_cpu_usage']
             ]
-            # Generate host_cpu_usage fake metrics
-            self.gnocchi.add_measures(metric_uuid, self._make_measures(3, 5))
+            if h['hypervisor_hostname'] in loaded_hosts:
+                cpu_measures = self._make_measures_host(10, 1, min=80, max=90)
+            else:
+                cpu_measures = self._make_measures_host(10, 1)
+            self.gnocchi.add_measures(cpu_metric_uuid, cpu_measures)
+
+            # Generate host_ram_usage fake metrics. Metric is based on
+            # hardware.memory.used which is in KB.
+            ram_metric_uuid = res['metrics'][
+                self.GNOCCHI_METRIC_MAP['host_ram_usage']
+            ]
+            if h['hypervisor_hostname'] in loaded_hosts:
+                mem_measures = self._make_measures_host(
+                    10, 1,
+                    min=int(h['memory_mb']) * 0.8 * 1024,
+                    max=int(h['memory_mb']) * 0.9 * 1024)
+            else:
+                mem_measures = self._make_measures_host(
+                    10, 1,
+                    min=int(h['memory_mb']) * 0.1 * 1024,
+                    max=int(h['memory_mb']) * 0.2 * 1024)
+            self.gnocchi.add_measures(ram_metric_uuid, mem_measures)
 
     def _show_measures(self, metric_uuid):
         try:
@@ -393,13 +591,18 @@ class BaseInfraOptimScenarioTest(manager.ScenarioTest):
         :param instance: Instance response body
         :param metrics: The metrics add to resource when using Gnocchi
         """
-        all_flavors = self.flavors_client.list_flavors()['flavors']
+        all_flavors = self.flavors_client.list_flavors(detail=True)['flavors']
         flavor_name = instance['flavor']['original_name']
         flavor = [f for f in all_flavors if f['name'] == flavor_name]
         if metrics == dict():
             metrics = {
                 self.GNOCCHI_METRIC_MAP["instance_cpu_usage"]: {
-                    'archive_policy_name': 'low'
+                    'archive_policy_name': 'ceilometer-low-rate',
+                    'unit': 'ns'
+                },
+                self.GNOCCHI_METRIC_MAP["instance_ram_usage"]: {
+                    'archive_policy_name': 'ceilometer-low',
+                    'unit': 'MB'
                 }
             }
         else:
@@ -411,7 +614,7 @@ class BaseInfraOptimScenarioTest(manager.ScenarioTest):
         resource_params = {
             'type': 'instance',
             'metrics': metrics,
-            'host': instance.get('OS-EXT-SRV-ATTR:hypervisor_hostname'),
+            'host': self.get_host_for_server(instance['id']),
             'display_name': instance.get('OS-EXT-SRV-ATTR:instance_name'),
             'image_ref': instance['image']['id'],
             'flavor_id': flavor[0]['id'],
@@ -419,18 +622,33 @@ class BaseInfraOptimScenarioTest(manager.ScenarioTest):
             'id': instance['id']
         }
         _, res = self.create_resource(**resource_params)
+
         # Generates cpu fake metrics
-        metric_uuid = res['metrics'][
+        cpu_metric_uuid = res['metrics'][
             self.GNOCCHI_METRIC_MAP["instance_cpu_usage"]
         ]
-        self.gnocchi.add_measures(metric_uuid, self._make_measures(3, 5))
+        cpu_measures = self._make_measures_instance(5, 5, metric_type='cpu')
+        self.gnocchi.add_measures(cpu_metric_uuid, cpu_measures)
 
-        self.assertTrue(test_utils.call_until_true(
-            func=functools.partial(
-                self._show_measures, metric_uuid),
-            duration=600,
-            sleep_for=2
-        ))
+        # Generate ram fake metrics
+
+        ram_metric_uuid = res['metrics'][
+            self.GNOCCHI_METRIC_MAP["instance_ram_usage"]
+        ]
+        ram_measures = self._make_measures_instance(
+            5, 5,
+            min=int(flavor[0]['ram']) * 0.8,
+            max=int(flavor[0]['ram']) * 0.9,
+            metric_type='ram')
+        self.gnocchi.add_measures(ram_metric_uuid, ram_measures)
+
+        for metric_uuid in [cpu_metric_uuid, ram_metric_uuid]:
+            self.assertTrue(test_utils.call_until_true(
+                func=functools.partial(
+                    self._show_measures, metric_uuid),
+                duration=600,
+                sleep_for=2
+            ))
 
     # ### PROMETHEUS ### #
     def _generate_prometheus_metrics(self,
@@ -441,7 +659,8 @@ class BaseInfraOptimScenarioTest(manager.ScenarioTest):
                                      interval_secs=30,
                                      add_unique_label=True,
                                      inc_factor=0.8,
-                                     start_value=1.0):
+                                     start_value=1.0,
+                                     timestamp=None):
         """Generates multiple samples for a given metric.
 
 
@@ -466,11 +685,11 @@ class BaseInfraOptimScenarioTest(manager.ScenarioTest):
           interval.
         :param start_value: value of the first sample to be
           generated.
+        :param timestamp: timestamp in ms for the most recent
 
         :return: String with all samples for a given metric.
         """
-        # timestamp needs to be in milliseconds
-        ts_now_ms = int(datetime.now().timestamp()*1000)
+        ts_now_ms = timestamp or int(datetime.now().timestamp()*1000)
 
         # NOTE(dviroel): by including a unique label value, we avoid the
         #  'out of order sample' error when pushing multiple
@@ -507,40 +726,111 @@ class BaseInfraOptimScenarioTest(manager.ScenarioTest):
         }
         # Generate cpu usage data for a instance
         # unit is ns, so for a 80%, inc_factor is 0.8 * 1e+9
-        data = self._generate_prometheus_metrics(
+        cpu_data = self._generate_prometheus_metrics(
             self.PROMETHEUS_METRIC_MAP['instance_cpu_usage'],
             labels=instance_labels,
             start_value=1.0,
             inc_factor=8e+8)
 
-        self.prometheus_client.add_measures(data)
+        # Generate memory usage data for a instance consuming 80%
+        # unit is megabytes, total is obtained from flavor
+        # no inc_factor as memory is saved as gauge
+        mem_usage_mb = int(instance['flavor']['ram'] * 0.8)
+        ram_data = self._generate_prometheus_metrics(
+            self.PROMETHEUS_METRIC_MAP['instance_ram_usage'],
+            metric_type='gauge',
+            labels=instance_labels,
+            start_value=mem_usage_mb,
+            inc_factor=0)
 
-    def make_host_statistic_prometheus(self):
-        """Create host resource and its measures in Prometheus. """
+        self.prometheus_client.add_measures(cpu_data)
+        self.prometheus_client.add_measures(ram_data)
+
+    def make_host_statistic_prometheus(self, loaded_hosts=[]):
+        """Create host resource and its measures in Prometheus.
+
+        :param loaded_hosts: list of hosts that we want to inject data
+          representing high usage of resource.
+        """
 
         hypervisors = self.get_hypervisors_setup()
 
         for h in hypervisors:
+            # When doing maths with prometheus, we need to
+            # have all metrics with the same timestamp.
+            timestamp = int(datetime.now().timestamp()*1000)
             instance = self.prometheus_client.prometheus_instances.get(
                 h['hypervisor_hostname'], None)
             if not instance:
                 LOG.info(f"Hostname {h['hypervisor_hostname']} does not "
                          "map to any prometheus instance.")
             else:
-                host_labels = {
+                # cpu metrics in prometheus are by cpu so we need to create
+                # a set of metrics for each one.
+                vcpus = h['vcpus']
+                for cpu in range(vcpus):
+                    host_labels = {
+                        "instance": instance,
+                        "fqdn": h['hypervisor_hostname'],
+                        "mode": "idle",
+                        "cpu": str(cpu),
+                    }
+                    # Generate host usage data
+                    # unit is seconds, that represent cpu in idle
+                    if h['hypervisor_hostname'] in loaded_hosts:
+                        cpu_data = self._generate_prometheus_metrics(
+                            self.PROMETHEUS_METRIC_MAP['host_cpu_usage'],
+                            labels=host_labels,
+                            start_value=1.0,
+                            inc_factor=0.0,
+                            timestamp=timestamp)
+                    else:
+                        cpu_data = self._generate_prometheus_metrics(
+                            self.PROMETHEUS_METRIC_MAP['host_cpu_usage'],
+                            labels=host_labels,
+                            start_value=1.0,
+                            inc_factor=1.0,
+                            timestamp=timestamp)
+                    self.prometheus_client.add_measures(cpu_data)
+
+                host_labels_ram = {
                     "instance": instance,
                     "fqdn": h['hypervisor_hostname'],
-                    "mode": "idle",
                 }
-                # Generate host usage data
-                # unit is seconds, that represent cpu in idle
-                data = self._generate_prometheus_metrics(
-                    self.PROMETHEUS_METRIC_MAP['host_cpu_usage'],
-                    labels=host_labels,
-                    start_value=1.0,
-                    inc_factor=0.4)
 
-                self.prometheus_client.add_measures(data)
+                # Generate memory usage data for a hypervisor
+                # simulate 80% of memory usage on loaded_hosts
+                # simulate 10% of memory load on others
+                # unit is megabytes, total is obtained from hypervisor
+                # no inc_factor as memory is saved as gauge
+
+                load = 0.8 if h['hypervisor_hostname'] in loaded_hosts else 0.1
+                mem_available_mb = int(h['memory_mb'] * (1 - load))
+                # metric is node_memory_MemAvailable_bytes which is in bytes
+                mem_available_bytes = mem_available_mb * 1024 * 1024
+                ram_data = self._generate_prometheus_metrics(
+                    self.PROMETHEUS_METRIC_MAP['host_ram_usage'],
+                    metric_type='gauge',
+                    labels=host_labels_ram,
+                    start_value=mem_available_bytes,
+                    inc_factor=0,
+                    timestamp=timestamp)
+                self.prometheus_client.add_measures(ram_data)
+
+                # Generate host total memory data for a hypervisor
+                # unit is megabytes, total is obtained from hypervisor
+                # no inc_factor as memory is saved as gauge
+                mem_total_mb = int(h['memory_mb'])
+                # metric is node_memory_MemTotal_bytes which is in bytes
+                mem_total_bytes = mem_total_mb * 1024 * 1024
+                ram_total_data = self._generate_prometheus_metrics(
+                    self.PROMETHEUS_METRIC_MAP['host_ram_total'],
+                    metric_type='gauge',
+                    labels=host_labels_ram,
+                    start_value=mem_total_bytes,
+                    inc_factor=0,
+                    timestamp=timestamp)
+                self.prometheus_client.add_measures(ram_total_data)
 
     # ### AUDIT TEMPLATES ### #
 
@@ -644,12 +934,15 @@ class BaseInfraOptimScenarioTest(manager.ScenarioTest):
                 return False
         return True
 
-    def execute_strategy(self, goal_name, strategy_name, **audit_kwargs):
+    def execute_strategy(self, goal_name, strategy_name,
+                         expected_actions=[], **audit_kwargs):
         """Execute an action plan based on the specific strategy
 
         - create an audit template with the specific strategy
         - run the audit to create an action plan
         - get the action plan
+        - Verify that all action types in the expected_actions
+          list are present in the action plan.
         - run the action plan
         - get results and make sure it succeeded
         """
@@ -696,6 +989,14 @@ class BaseInfraOptimScenarioTest(manager.ScenarioTest):
         action_plan = action_plans['action_plans'][0]
 
         _, action_plan = self.client.show_action_plan(action_plan['uuid'])
+        created_actions = self.client.list_actions(
+            action_plan_uuid=action_plan["uuid"])[1]['actions']
+
+        if expected_actions:
+            action_types = {a['action_type'] for a in created_actions}
+            if set(expected_actions) != action_types:
+                self.fail("The audit has found action types %s when expecting "
+                          "%s" % (action_types, expected_actions))
 
         if action_plan['state'] in ('SUPERSEDED', 'SUCCEEDED'):
             # This means the action plan is superseded so we cannot trigger it,
@@ -734,23 +1035,40 @@ class BaseInfraOptimScenarioTest(manager.ScenarioTest):
             return False
 
     def wait_for_instances_in_model(self, instances, timeout=300):
-        """Waits until all instance ids are mapped to a model."""
+        """Waits until all instance ids are mapped to a model.
+
+        Get the model and save instance ids and hypervisor hostname
+        Get instances details and save instance ids and current hypervisora
+        hostname
+        (hypervisor hostname from argument is the one where it was created)
+        Compare the two lists and wait until they are equal.
+        """
+
         timeout_end = time.time() + timeout
 
         _, body = self.client.list_data_models(data_model_type="compute")
-        model_uuids = [s["server_uuid"]
-                       for s in body.get("context", []) if "server_uuid" in s]
+        model_pairs = [(s['server_uuid'], s['node_hostname'])
+                       for s in body.get('context', [])
+                       if 'server_uuid' in s and 'node_hostname' in s]
+        instance_pairs = []
+        for instance in instances:
+            s = self.mgr.servers_client.show_server(instance['id'])['server']
+            instance_pairs.append((s['id'], s['OS-EXT-SRV-ATTR:host']))
 
-        ids = [instance['id'] for instance in instances]
-        while not set(ids) <= set(model_uuids):
+        # If no instances were created, we should not wait for them
+        if not instance_pairs:
+            raise Exception("No instances were created.")
+
+        # Check all instances are in the model and model is not empty
+        while (not set(instance_pairs) <= set(model_pairs) or not model_pairs):
             time.sleep(15)
             if time.time() >= timeout_end:
                 raise Exception("Instances are not mapped to compute model.")
 
             _, body = self.client.list_data_models(data_model_type="compute")
-            model_uuids = [
-                s["server_uuid"]
-                for s in body.get("context", []) if "server_uuid" in s]
+            model_pairs = [(s['server_uuid'], s['node_hostname'])
+                           for s in body.get('context', [])
+                           if 'server_uuid' in s and 'node_hostname' in s]
 
     def wait_delete_instances_from_model(self, timeout=300):
         """Waits until all deleted instaces be removed from model."""
