@@ -16,6 +16,7 @@
 # limitations under the License.
 
 from tempest import config
+from tempest.lib import decorators
 
 from watcher_tempest_plugin.tests.scenario import base
 
@@ -25,6 +26,11 @@ CONF = config.CONF
 class TestExecuteStrategies(base.BaseInfraOptimScenarioTest):
     """Tests for strategies"""
 
+    # Minimal version required for list data models
+    min_microversion = "1.3"
+    # Minimal version required for _create_one_instance_per_host
+    compute_min_microversion = base.NOVA_API_VERSION_CREATE_WITH_HOST
+
     @classmethod
     def skip_checks(cls):
         super(TestExecuteStrategies, cls).skip_checks()
@@ -39,18 +45,18 @@ class TestExecuteStrategies(base.BaseInfraOptimScenarioTest):
             raise cls.skipException("Live migration is not enabled")
 
         cls.wait_for_compute_node_setup()
-        cls.initial_compute_nodes_setup = cls.get_compute_nodes_setup()
-        enabled_compute_nodes = [cn for cn in cls.initial_compute_nodes_setup
-                                 if cn.get('status') == 'enabled']
+        enabled_compute_nodes = cls.get_enabled_compute_nodes()
         if len(enabled_compute_nodes) < 2:
             raise cls.skipException(
                 "Less than 2 compute nodes are enabled, "
                 "skipping multinode tests.")
 
+    @decorators.idempotent_id('62766a61-dfc4-478c-b80b-86d871227e67')
     def test_execute_basic_strategy(self):
         self.addCleanup(self.rollback_compute_nodes_status)
         self.addCleanup(self.wait_delete_instances_from_model)
-        instances = self._create_one_instance_per_host_with_statistic()
+        self.addCleanup(self.clean_injected_metrics)
+        instances = self._create_one_instance_per_host()
         # wait for compute model updates
         self.wait_for_instances_in_model(instances)
         self.make_host_statistic()
@@ -68,31 +74,17 @@ class TestExecuteStrategies(base.BaseInfraOptimScenarioTest):
 
         self.execute_strategy(goal_name, strategy_name, **audit_kwargs)
 
+    @decorators.idempotent_id('a660e484-da76-478c-b18c-2eff243dde48')
     def test_execute_dummy_strategy(self):
         goal_name = "dummy"
         strategy_name = "dummy"
         audit_kwargs = dict()
 
-        self.execute_strategy(goal_name, strategy_name, **audit_kwargs)
-
-    def test_execute_host_maintenance_strategy(self):
-        self.addCleanup(self.rollback_compute_nodes_status)
-        self.addCleanup(self.wait_delete_instances_from_model)
-        instances = self._create_one_instance_per_host_with_statistic()
-        hostname = instances[0].get('OS-EXT-SRV-ATTR:hypervisor_hostname')
-        # wait for compute model updates
-        self.wait_for_instances_in_model(instances)
-
-        goal_name = "cluster_maintaining"
-        strategy_name = "host_maintenance"
-        audit_kwargs = {
-            "parameters": {
-                "maintenance_node": hostname
-            }
-        }
-
-        self.execute_strategy(goal_name, strategy_name, **audit_kwargs)
+        self.execute_strategy(goal_name, strategy_name,
+                              expected_actions=['sleep', 'nop'],
+                              **audit_kwargs)
 
+    @decorators.idempotent_id('7e3a9195-acc5-40cf-96da-a0a2883294d3')
     def test_execute_storage_capacity_balance_strategy(self):
         self.addCleanup(self.rollback_compute_nodes_status)
         self.create_volume(imageRef=CONF.compute.image_ref, size=3)
@@ -107,19 +99,25 @@ class TestExecuteStrategies(base.BaseInfraOptimScenarioTest):
 
         self.execute_strategy(goal_name, strategy_name, **audit_kwargs)
 
+    @decorators.idempotent_id('9f3ee978-e033-4c1e-bbf2-9e5a5cb8a365')
     def test_execute_vm_workload_consolidation_strategy(self):
+        # This test requires metrics injection
+
         self.addCleanup(self.rollback_compute_nodes_status)
         self.addCleanup(self.wait_delete_instances_from_model)
+        self.addCleanup(self.clean_injected_metrics)
         metrics = {
             'instance_cpu_usage': {},
             'instance_ram_usage': {},
             'instance_ram_allocated': {},
             'instance_root_disk_size': {},
         }
-        instances = self._create_one_instance_per_host_with_statistic(metrics)
+        instances = self._create_one_instance_per_host()
         # wait for compute model updates
         self.wait_for_instances_in_model(instances)
         self.make_host_statistic()
+        for instance in instances:
+            self.make_instance_statistic(instance, metrics=metrics)
 
         goal_name = "server_consolidation"
         strategy_name = "vm_workload_consolidation"
@@ -131,19 +129,29 @@ class TestExecuteStrategies(base.BaseInfraOptimScenarioTest):
 
         self.execute_strategy(goal_name, strategy_name, **audit_kwargs)
 
-    def test_execute_workload_stabilization_strategy(self):
+    @decorators.idempotent_id('14360d59-4923-49f7-bfe5-31d6a819b6f7')
+    def test_execute_workload_stabilization_strategy_cpu(self):
+        # This test requires metrics injection
+
         self.addCleanup(self.rollback_compute_nodes_status)
         self.addCleanup(self.wait_delete_instances_from_model)
-        instances = self._create_one_instance_per_host_with_statistic()
-        self._pack_all_created_instances_on_one_host(instances)
+        self.addCleanup(self.clean_injected_metrics)
+        host = self.get_enabled_compute_nodes()[0]['host']
+        instances = []
+        for _ in range(2):
+            instance = self._create_instance(host=host)
+            instances.append(instance)
         # wait for compute model updates
         self.wait_for_instances_in_model(instances)
+        self.make_host_statistic(loaded_hosts=[host])
+        for instance in instances:
+            self.make_instance_statistic(instance)
 
         audit_parameters = {
             "metrics": ["instance_cpu_usage"],
-            "thresholds": {"instance_cpu_usage": 0.2},
+            "thresholds": {"instance_cpu_usage": 0.05},
             "weights": {"instance_cpu_usage_weight": 1.0},
-            "periods": {"instance": 72000, "compute_node": 60000},
+            "periods": {"instance": 400, "compute_node": 300},
             "instance_metrics": {"instance_cpu_usage": "host_cpu_usage"},
             "granularity": 300,
             "aggregation_method": {"instance": "mean", "compute_node": "mean"}}
@@ -152,35 +160,48 @@ class TestExecuteStrategies(base.BaseInfraOptimScenarioTest):
         strategy_name = "workload_stabilization"
         audit_kwargs = {"parameters": audit_parameters}
 
-        self.execute_strategy(goal_name, strategy_name, **audit_kwargs)
+        self.execute_strategy(goal_name, strategy_name,
+                              expected_actions=['migrate'], **audit_kwargs)
+
+    @decorators.idempotent_id('4988b894-b237-4ebc-9af1-ecf1f9ea734e')
+    def test_execute_workload_stabilization_strategy_ram(self):
+        # This test requires metrics injection
 
-    def test_execute_zone_migration_live_migration_strategy(self):
         self.addCleanup(self.rollback_compute_nodes_status)
         self.addCleanup(self.wait_delete_instances_from_model)
-        instances = self._create_one_instance_per_host_with_statistic()
-        node = instances[0].get('OS-EXT-SRV-ATTR:hypervisor_hostname')
+        self.addCleanup(self.clean_injected_metrics)
+        host = self.get_enabled_compute_nodes()[0]['host']
+        instances = []
+        for _ in range(2):
+            instance = self._create_instance(host=host)
+            instances.append(instance)
+
         # wait for compute model updates
         self.wait_for_instances_in_model(instances)
-
-        vacant_node = [hyp['hypervisor_hostname'] for hyp
-                       in self.get_hypervisors_setup()
-                       if hyp['state'] == 'up'
-                       and hyp['hypervisor_hostname'] != node][0]
+        self.make_host_statistic(loaded_hosts=[host])
+        for instance in instances:
+            # Inject metrics after the instances are created
+            self.make_instance_statistic(instance)
 
         audit_parameters = {
-            "compute_nodes": [{"src_node": node, "dst_node": vacant_node}],
-            }
+            "metrics": ["instance_ram_usage"],
+            "thresholds": {"instance_ram_usage": 0.05},
+            "periods": {"instance": 400, "compute_node": 300}}
 
-        goal_name = "hardware_maintenance"
-        strategy_name = "zone_migration"
+        goal_name = "workload_balancing"
+        strategy_name = "workload_stabilization"
         audit_kwargs = {"parameters": audit_parameters}
 
-        self.execute_strategy(goal_name, strategy_name, **audit_kwargs)
+        self.execute_strategy(goal_name, strategy_name,
+                              expected_actions=['migrate'], **audit_kwargs)
 
+    @decorators.idempotent_id('c0c061e9-4713-4a23-a6e1-5db794add685')
     def test_execute_node_resource_consolidation_strategy_with_auto(self):
+        # This test does not require metrics injection
+
         self.addCleanup(self.rollback_compute_nodes_status)
         self.addCleanup(self.wait_delete_instances_from_model)
-        instances = self._create_one_instance_per_host_with_statistic()
+        instances = self._create_one_instance_per_host()
         # wait for compute model updates
         self.wait_for_instances_in_model(instances)
 
@@ -194,10 +215,13 @@ class TestExecuteStrategies(base.BaseInfraOptimScenarioTest):
 
         self.execute_strategy(goal_name, strategy_name, **audit_kwargs)
 
+    @decorators.idempotent_id('12312f2b-ff7a-4722-9aa3-0262608e1ef0')
     def test_execute_node_resource_consolidation_strategy_with_specify(self):
+        # This test does not require metrics injection
+
         self.addCleanup(self.rollback_compute_nodes_status)
         self.addCleanup(self.wait_delete_instances_from_model)
-        instances = self._create_one_instance_per_host_with_statistic()
+        instances = self._create_one_instance_per_host()
         # wait for compute model updates
         self.wait_for_instances_in_model(instances)
 
